// Copyright 2016 The go-ethereum Authors
// This file is part of the go-ethereum library.
//
// The go-ethereum library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// The go-ethereum library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with the go-ethereum library. If not, see <http://www.gnu.org/licenses/>.

package nodecfg

import (
	"github.com/ledgerwatch/erigon/common/paths"
	"github.com/ledgerwatch/erigon/node/nodecfg/datadir"
	"github.com/ledgerwatch/erigon/p2p"
	"github.com/ledgerwatch/erigon/p2p/nat"
	"github.com/ledgerwatch/erigon/rpc/rpccfg"
)

const (
	DefaultHTTPHost       = "localhost" // Default host interface for the HTTP RPC server
	DefaultHTTPPort       = 8545        // Default TCP port for the HTTP RPC server
	DefaultEngineHTTPPort = 8551        // Default TCP port for the engineApi HTTP RPC server
	DefaultWSHost         = "localhost" // Default host interface for the websocket RPC server
	DefaultWSPort         = 8546        // Default TCP port for the websocket RPC server
	DefaultGRPCHost       = "localhost" // Default host interface for the GRPC server
	DefaultGRPCPort       = 8547        // Default TCP port for the GRPC server
)

// DefaultConfig contains reasonable default settings.
var DefaultConfig = Config{
	Dirs:             datadir.New(paths.DefaultDataDir()),
	HTTPPort:         DefaultHTTPPort,
	HTTPModules:      []string{"net", "web3"},
	HTTPVirtualHosts: []string{"localhost"},
	HTTPTimeouts:     rpccfg.DefaultHTTPTimeouts,
	WSPort:           DefaultWSPort,
	WSModules:        []string{"net", "web3"},
	P2P: p2p.Config{
		ListenAddr:      ":30303",
		ListenAddr65:    ":30304",
		MaxPeers:        100,
		MaxPendingPeers: 1000,
		NAT:             nat.Any(),
	},
}
